"use strict";
/*
  Writers and Searchers should throw errors of this class in order for
  the server to generate friendly, JSONAPI error responses.

  The arguments to this class are defined via
  http://jsonapi.org/format/#error-objects

*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const logger_1 = __importDefault(require("@cardstack/logger"));
const http_1 = require("http");
const log = logger_1.default('cardstack/error');
class E extends Error {
    constructor(detail, { status, title, source } = {}) {
        super(detail);
        this.detail = detail;
        this.status = status || 500;
        this.title = title || http_1.STATUS_CODES[this.status];
        this.source = source;
        this.isCardstackError = true;
        this.additionalErrors = null;
    }
    toJSON() {
        return {
            title: this.title,
            detail: this.detail,
            code: this.status,
            source: this.source
        };
    }
    static async withJsonErrorHandling(ctxt, fn) {
        try {
            return await fn();
        }
        catch (err) {
            if (!err.isCardstackError) {
                throw err;
            }
            if (err.status === 500) {
                log.error(`Unexpected error: ${err.status} - ${err.message}\n${err.stack}`);
            }
            let errors = [err];
            if (err.additionalErrors) {
                errors = errors.concat(err.additionalErrors);
            }
            ctxt.body = { errors };
            ctxt.status = errors[0].status;
        }
    }
}
module.exports = E;
//# sourceMappingURL=error.js.map